"""
Модуль объединяет в себе несколько функций для получения различного
рода IP-адресов.
В данном случае это:
- локальный IP;
- IP-роутера(маршрутизатора);
- внешний IP-адрес.
Для работы данной функции нужно установить библиотеку requests:
pip install requests
"""
from platform import system
from subprocess import check_output
from socket import gethostbyname, socket, AF_INET, SOCK_DGRAM

from requests import exceptions, get


class IPDefinition:
    def __init__(self):
        """
        Класс для получения IP-адресов.
        self.local_ip: локальный IP-адрес. Получает значение из функции.
        self.public_ip: внешний IP-адрес. Получает значение из функции.
        self.router_ip: IP-адрес роутера или маршрутизатора. Получает значение из функции.
        """
        self.local_ip = local()
        self.public_ip = public()
        self.router_ip = router()


def local():
    """
    Получаем локальный IP-адрес с помощью коннекта
    на адрес 10.255.255.255. В ответ получаем имя
    сокета, которое и является адресом.
    :return: возвращает локальный IP-адрес.
    """
    st = socket(AF_INET, SOCK_DGRAM)
    try:
        st.connect(('10.255.255.255', 1))
        ip = st.getsockname()[0]
    except Exception:
        ip = '127.0.0.1'
    finally:
        st.close()
    return ip


def public():
    """
    Получение внешнего, публичного IP-адреса.
    :return: возвращает значение публичного IP-адреса.
    """
    try:
        return get('https://api.ipify.org/').text
    except exceptions.ConnectionError:
        return '127.0.0.1'


def router():
    """
    Получение IP-адреса роутера или маршрутизатора.
    :return: возвращает IP-адрес роутера или маршрутизатора.
    """
    ip_route = None
    if system() == "Linux":
        ip_route = str(check_output('route -n | grep UG', shell=True).decode()).split()[1]
    elif system() == "Windows":
        com = 'route PRINT 0* -4 | findstr 0.0.0.0'.split()
        interface_temp = check_output(com, shell=True).decode('cp866')
        ip_route = interface_temp.split()[-3]

    if ip_route.isdigit():
        return ip_route
    else:
        sock = gethostbyname(ip_route)
        return sock


def geo_ip(ip):
    """
    Функция для получения координат по внешнему IP-адресу.
    Выполняет обращение к сервису, в который передает адрес.
    В ответ получает JSON с координатами и не только (остальные данные не возвращаются из функции).
    :param ip: внешний IP-адрес или любой адрес координаты которого нужно узнать.
    :return: возвращает широту и долготу по IP.
    """
    try:
        url = f'http://ip-api.com/json/{ip}'
        req = get(url=url).json()
        return [req['lat'], req['lon']]
    except Exception:
        return 'Не удалось получить координаты'

geo_ip(IPDefinition().public_ip)