"""
Скрипт для пинга сети, проверки наличия в ней компьютеров
и сканирование каждого из них на наличие общедоступных ресурсов.
Для работы требует установки библиотеки ping3: pip install ping3.
Также, для работы скрипта на компьютере необходимо наличие
установленного пакета smbclient. Если данного пакета не установлено,
пользователю предлагается его установить.
Также в скрипт импортируются модули для работы с IP-адерсами, то есть
для определения локального, внешнего адреса, а также адреса роутера-маршрутизатора и
модуль для работы с smbclient и сканирования сети с его помощью.
"""

from shutil import which
from socket import socket, AF_INET, SOCK_STREAM
from subprocess import Popen, PIPE
from threading import Thread
from ipaddress import ip_network

from ping3 import ping

from definition import IPDefinition
from smb_scan import SMBscan

IP_SET = []


def ping_net(ip):
    """
    Проверяет доступность компьютера в сети с помощью
    пинга, если доступен, добавляет адрес в глобальный
    список, если нет, выход из функции.
    :param ip: IP-адрес для пинга компьютеро.
    :return: выход из функции.
    """
    try:
        if ping(ip):
            if ip == IPDefinition().local_ip:
                return
            elif ip == IPDefinition().router_ip:
                return
            IP_SET.append(ip)
    except PermissionError:
        return


def thread_func(target):
    """
    Запуск потоков для пинга адресов в диапазоне, который
    итерируется с помощью ip_network. Ожидание завершения потоков.
    :param target: IP-адрес с маской сети.
    :return: выход из функции.
    """
    threads = []

    try:
        for ip in ip_network(target, False):
            t = Thread(target=ping_net, kwargs={'ip': str(ip)})
            t.daemon = True
            t.start()
            threads.append(t)

        for num, thread in enumerate(threads):
            thread.join()
    except ValueError:
        print('Неверно указан адрес или маска. Повторите ввод')
        return


def check_port(ip, port=139):
    """
    Проверка порта службы SMB.
    :param ip: IP-адрес для проверки порта.
    :param port: порт сеансовой службы NetBIOS, которая
    активизирует браузер поиска других компьютеров,
    службы совместного использования файлов, Net Logon и службу сервера.
    По умолчанию 139.
    :return: возвращает True, если порт открыт и
    False, если порт недоступен.
    """
    s = socket(AF_INET, SOCK_STREAM)
    s.settimeout(2)
    try:
        s.connect((ip, int(port)))
        s.shutdown(2)
        return True
    except Exception:
        return False


def smb_install():
    """
    Проверка наличия smbclient на компьютере.
    Если он не установлен в системе, пользователю
    будет предложено его установить. Запрашивается пароль
    для установки.
    """
    if not which('smbclient'):
        sudo_password = input("Внимание!\nНе установлен smbclient.\nДля его установки введите пароль sudo: ")
        command1 = 'apt install smbclient -y'.split()
        p = Popen(['sudo', '-S'] + command1, stdin=PIPE, stderr=PIPE,
                  universal_newlines=True)
        p.communicate(sudo_password + '\n')


def network_scan(ip_addr, user_pass):
    """
    Сканирование сети на наличие в ней компьютеров,
    сканирование на каждом из них наличие общедоступных ресурсов.
    Вывод полученных данных в терминал.
    :param ip_addr: IP-адрес и маска сети.
    :param user_pass: логин и пароль пользователя в виде: login%pass
    """
    set_smb = []
    thread_func(ip_addr)
    if len(IP_SET) > 0:
        for ip in IP_SET:
            if check_port(ip):
                set_smb.append(ip)
        print(f'\nНайдено компьютеров с SMB: {len(set_smb)}/{len(IP_SET)}')
        if len(set_smb) > 0:
            for cln in set_smb:
                print(f'\n[+] Сканирую: {cln}\n{"-" * 50}')
                print(f'{SMBscan(cln, user_pass).smb_get()}')
                print(SMBscan(cln, user_pass).smb_access())
        else:
            print('Сканировать нечего')
    else:
        print('Компьютеров в сети не обнаружено')


def main():
    """
    Проверка пользовательского выбора. В зависимости
    от выбора выполняется пинг сети, сканирование адреса или
    сканирование всей сети.
    """
    smb_install()
    user_ch = input('\nВыберите действие:\n  [1] Пинг сети\n  [2] Сканирование одного компьютера\n  '
                    '[3] Сканирование сети\n  [4] Выход\n  >>> ')
    if user_ch == "1":
        ip_mask = input("\nВведите IP и маску сети (192.168.1.1/24): ")
        thread_func(ip_mask)
        if len(IP_SET) > 0:
            print('\nНайдены компьютеры:')
            for ip in IP_SET:
                print(f'    {ip}')
        else:
            print('Компьютеров не найдено')
    elif user_ch == "2":
        ip = input('Введите IP адрес: ')
        user = input('Введите логин администратора, если он вам известен.\nЕсли нет, нажмите Enter: ')
        if user != "":
            pwd = input("Введите пароль администратора, если она вам известен.\nЕсли нет, нажмите Enter: ")
            user_pass = f"-U {user}%{pwd}"
        else:
            user_pass = ""
        if check_port(ip):
            print(SMBscan(ip, user_pass).smb_get())
            print(SMBscan(ip, user_pass).smb_access())
    elif user_ch == "3":
        ip_mask = input("\nВведите IP и маску сети (192.168.1.1/24): ")
        user = input('Введите логин администратора, если он вам известен.\nЕсли нет, нажмите Enter: ')
        if user != "":
            pwd = input("Введите пароль администратора, если она вам известен.\nЕсли нет, нажмите Enter: ")
            user_pass = f"-U {user}%{pwd}"
        else:
            user_pass = ""
        network_scan(ip_mask, user_pass)
    elif user_ch == "4":
        print('Good by...')
        exit(0)


if __name__ == "__main__":
    main()
